<?php
class Xoogu_CurrencyConversion_Admin extends Xoogu_CurrencyConversion{
    
    public $mainPluginFile;
    
    /**
     *Initialise the plugin
     */
    public function __construct(){
        parent::__construct();
        //register the activation and deactivation methods
        $mainPluginFile=$this->mainPluginFile=dirname(__FILE__).'/xoogu-currency-conversion.php';
        register_activation_hook( $mainPluginFile, array($this, 'activate'));
        register_deactivation_hook( $mainPluginFile, array($this, 'deactivate'));
        register_uninstall_hook( $mainPluginFile, array('Xoogu_CurrencyConversion_Admin', 'uninstall'));
        //multisite new blog added
        add_action( 'wpmu_new_blog', 'new_site_added', 10, 6);
        
        //action hook for the cron
        add_action('xoogu_currency_conversion_update', array($this, 'updateRates'));
        
        //on post save
        add_action( 'save_post', array($this, 'onPostSave'), 9999 );
        
        //add admin menu
        add_action('admin_menu', array($this, 'add_admin_menu'));
		//add settings page link in plugins list
        add_filter('plugin_action_links_'.plugin_basename($mainPluginFile), array($this, 'plugin_action_links'));
        load_plugin_textdomain('xoogu-currency-conversion', false, dirname(plugin_basename(__FILE__)).'/lang/');
		
		//$this->updateRates();
        
    }
    
    
    /**
     *Adds the plugin settings page to the Settings menu
     */
    public function add_admin_menu(){
		//Add a new menu item under Settings
        $pageTitle=sprintf(__('%s Currency Conversion','xoogu-currency-conversion'), 'Xoogu');
		$hook_suffix = add_options_page( $pageTitle, $pageTitle, 'manage_options', 'xoogu_currency_conversion', array($this, 'display_page') );
		//Using registered $page handle to hook stylesheet loading
        add_action( 'load-' . $hook_suffix, array($this, 'admin_load') );
    }
    
    
    /**
     *Load scripts, css etc. on the settings page
     */
    public function admin_load(){
        wp_enqueue_style('xoogu_currency_conversion_admin', plugins_url('admin.css', __FILE__), null, self::$version);
        wp_enqueue_script('xoogu_currency_conversion_admin', plugins_url('admin.js', __FILE__), array('jquery'), self::$version);
        wp_localize_script( 'xoogu_currency_conversion_admin', 'objectL10n', array(
			'show_opts'  => __('Show advanced options', 'xoogu-currency-conversion' ),
			'hide_opts' => __('Hide advanced options', 'xoogu-currency-conversion' )
		) );
    }
    
    
    /**
     *Validate and save the POSTed settings when the settings form is submitted
     */
    public function validate_settings($input){
        
        //check nonce & referrer
        if(wp_verify_nonce($_POST['_xoogu_currencyConversion_nonce'], 'save_settings') !== 1){
            wp_die(__('A valid nonce was not found.','xoogu-currency-conversion') );
        }
        if(!current_user_can('manage_options')){
            wp_die(__('Sorry, but you must have the manage_options capability to view and edit settings. See <a href="http://codex.wordpress.org/Roles_and_Capabilities">Wordpress Codex: Roles and Capabilities</a> for more information.','xoogu-currency-conversion') );
        }
        $errors = array();
        
        //from
        if(!empty($input['from'])){
            if($this->getCurrencyByCode($input['from'])){
                $this->options['from']=(string)$input['from'];
            }else{
                $errors['from'] = sprintf(__('%s is not a valid currency code', 'xoogu-currency-conversion'), $input['from']);
            }
        }else{
            unset($this->options['from']);
        }
        
        //dec_places
        if($input['dec_places'] !== ''){
            if(!$this->isValidInteger($input['dec_places']) || (int)$input['dec_places']<0){
                $errors['dec_places'] = 'dec_places must be an integer of zero or greater (or blank)';
            }else{
                $this->options['dec_places']=(int)$input['dec_places'];
            }
        }else{
            unset($this->options['dec_places']);
        }
        
        //static
        if(!empty($input['static'])){
            $this->options['static']=1;
        }else{
            unset($this->options['static']);
        }
        
        //updateFreq
        $freqs = wp_get_schedules();
        if(empty($freqs[$input['updateFreq']])){
            $errors['updateFreq'] = sprintf(__('%s is not a valid update frequency', 'xoogu-currency-conversion'), $input['updateFreq']);
        }else{
            $this->options['updateFreq']=(string)$input['updateFreq'];
        }
        
        //format
        $this->options['format']=(string)$input['format'];
        
        //geoFunc
        if(!empty($input['geoFunc'])){
            $geoFuncs = $this->detectGeoIP();
            if(empty($geoFuncs[$input['geoFunc']])){
                $errors['geoFunc']= sprintf(__('%s is not a valid Geo IP method for this site','xoogu-currency-conversion'), $input['geoFunc']);
            }
            else{
                $this->options['geoFunc']=(string)$input['geoFunc'];
            }
        }
        else{
            unset($this->options['geoFunc']);
        }
        
        //addOns
        if(!empty($input['addOns'])){
            //activate any addons set to on
            foreach($input['addOns'] as $addOn => $val){
                $this->options['addOns'][$addOn] = true;
            }
        }
        if(!empty($this->options['addOns'])){
            //deactivate any previously active addons now set to off
            foreach($this->options['addOns'] as $addOn => $val){
                if(!$input['addOns'][$addOn]){
                    unset($this->options['addOns'][$addOn]);
                }
            }
        }
        
        //should be impossible to have any errors apart from dec_places not being an int
        if(!empty($errors)){
            return $errors;
        }
        update_option('xoogu_currency_conversion', $this->options);
    }
    
    
    /**
     *Display the settings page
     *See also the duplicate in Xoogu_CurrencyConversion_Admin_MCE::display_mce_form
     */
    public function display_page(){
        if(!current_user_can('manage_options')){
            wp_die(__('Sorry, but you must have the manage_options capability to view and edit settings. See <a href="http://codex.wordpress.org/Roles_and_Capabilities">Wordpress Codex: Roles and Capabilities</a> for more information.','xoogu-currency-conversion') );
        }
        //if the form was submitted, process the POST
        if(isset($_POST['submit']) || isset($_POST['installGeoIP'])){
            if(isset($_POST['submit'])){
                $errors = $this->validate_settings($_POST['options']);
                if(empty($errors)){
                    ?><div class="updated"><p><?php _e('Settings saved successfully.','xoogu-currency-conversion')?></p></div><?php
                }
            }
            elseif(isset($_POST['installGeoIP'])){
                //try to install the GeoIP plugin
                if( !($geoIPinstalled = $this->_installGeoIP()) ){
                    //the function will return false if the filesystem credentials form needs to be displayed
                    //in which case, we don't need to display our settings form.
                    return;
                }
                elseif( is_wp_error($geoIPinstalled) ){
                    $errors = $res->get_error_messages();
                } 
                elseif($geoIPinstalled === true){
                 ?><div class="updated"><p><?php _e('GeoIP Detection plugin installed and activated successfully.','xoogu-currency-conversion')?></p></div><?php
                }
            }
            //set the form values to those that were submitted
            $formValues = $_POST['options'];
        }
        else{
            //otherwise set the form values to the saved options
            $formValues = $this->options;
        }
        ?>
        <h1><?php printf( __('%s Currency Conversion Plugin Settings','xoogu-currency-conversion'), 'Xoogu');?></h1>
        <form action="<?php echo get_admin_url(null, 'options-general.php?page=xoogu_currency_conversion');?>" method="post" enctype="multipart/form-data" autocomplete="off" id="xoogu_currency_conversion_options_form">
            <?php
            //if there were any errors when we validated the POSTed data, print them out
            if(!empty($errors)){
                echo '<div class="error"><ul>';
                foreach($errors as $msg){
                    echo '<li>'.$msg.'</li>';
                }
                echo '</ul></div>';
            }?>
            <dl>
                <dt><?php _e('Default currency to convert from','xoogu-currency-conversion');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('You can set a default currency to convert from. This will always be used for conversions unless you specify the from attribute in the shortcode tag.','xoogu-currency-conversion');?></p>
                        <p><?php _e('If left blank, then the shortcode will try to determine which currency to convert from automatically based on the text it encloses. (Or it will use the from attribute if this is set).','xoogu-currency-conversion');?></p>
                    </div>
                    <label for="from"><?php _e('From:','xoogu-currency-conversion');?></label>
                    <select name="options[from]" id="from">
                        <?php $codes = $this->getCurrencyList();
                        array_unshift($codes, '');
                        foreach($codes as $code){
                            echo '<option'.(!empty($formValues['from']) && $formValues['from']==$code ? ' selected="selected"' : '').'>'.$code.'</option>';
                        }
                        unset($code,$codes);?>
                    </select>
                </dd>
                <dt><?php _e('Detect user\'s country from their IP address','xoogu-currency-conversion');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('Unless the currency to convert to is given as a shortcode attribute, the shortcode will convert the amount to the currency of the visitor\'s country. To do this, the plugin needs a way of looking up the visitor\'s country from their IP address.','xoogu-currency-conversion');?></p>
                        <p><?php _e('The plugin supports a number of different Geo IP lookup methods, and you can choose which one to use. If the drop-down below has no options to choose from, you can install the GeoIP Detection plugin to add the needed functionality.','xoogu-currency-conversion');?></p>
                    </div>
                    <?php
                    $geoFuncs = $this->detectGeoIP();
                    //display the button to install the GeoIP Detection plugin
                    if(current_user_can('activate_plugins') && empty($geoFuncs)){
                        //we don't network activate the geoIP plugin, so a note to tell super admin this is the case
                        if(is_multisite() && is_super_admin()){
                            echo '<p>'.__('Please note that installing the GeoIP Detection plugin via this button only activates the plugin for the current blog. You will need to manually network activate the GeoIP Detection plugin if you want it to be used for all sites in the network.','xoogu-currency-conversion').'</p>';
                        }
                        //button to auto install and activate the GeoIP Detection plugin - see $this->_installGeoIP()
                        submit_button( __('Install GeoIP Detection plugin','xoogu-currency-conversion'), 'secondary', 'installGeoIP' );
                        //text link so user can download and install the GeoIP plugin manually if they wish
                        echo '<p>'.sprintf(__('Or download and install manually from %s.','xoogu-currency-conversion'),'<a href="http://wordpress.org/plugins/geoip-detect/">http://wordpress.org/plugins/geoip-detect/</a>').'</p>';
                    }
                    ?>
                    <label for="geoFunc"><?php _e('Geo IP method:','xoogu-currency-conversion');?></label>
                    <select id="geoFunc" name="options[geoFunc]">
                    <?php
                    array_unshift($geoFuncs, '');
                    foreach($geoFuncs as $key => $name){
                        echo '<option value="'.$key.'"'.($formValues['geoFunc']==$key ? ' selected="selected"' : '').'>'.$name.'</option>';
                    }
                    ?></select>
                    <p><?php if(!empty($formValues['geoFunc'])){
                        $info = $this->getCurrencyByIP();
                        if(!$info){
                            printf(__('Couldn\'t detect currency details for your IP address %s', 'xoogu-currency-conversion'), $_SERVER['REMOTE_ADDR']);
                        }else{
                            printf(__('Your country code is %s, which uses the currency code %s', 'xoogu-currency-conversion'), $info->country, $info->code);
                        }
                        unset($info);
                    }?></p>
                </dd>
                <dt><?php _e('Replacement format','xoogu-currency-conversion');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('The format that will be used for replacing the text enclosed by the shortcode. You can use the following replacements as part of the format:','xoogu-currency-conversion');?></p>
                        <dl>
                            <dt>%o</dt>
                            <dd><?php _e('Original text (the text enclosed by the shortcode)','xoogu-currency-conversion');?></dd>
                            <dt>%sy</dt>
                            <dd><?php _e('Currency symbol e.g. $','xoogu-currency-conversion');?></dd>
                            <dt>%sn</dt>
                            <dd><?php _e('Currency symbol in html numerical entity format','xoogu-currency-conversion');?></dd>
                            <dt>%sx</dt>
                            <dd><?php _e('Currency symbol in html hex entity format','xoogu-currency-conversion');?></dd>
                            <dt>%n</dt>
                            <dd><?php _e('Name of currency e.g. dollars','xoogu-currency-conversion');?></dd>
                            <dt>%fn</dt>
                            <dd><?php _e('Full name of currency e.g. United States Dollars','xoogu-currency-conversion');?></dd>
                            <dt>%c</dt>
                            <dd><?php _e('ISO currency code e.g. USD','xoogu-currency-conversion');?></dd>
                            <dt>%a</dt>
                            <dd><?php _e('The amount converted to local currency','xoogu-currency-conversion');?></dd>
                            <dt>%asy</dt>
                            <dd><?php _e('The amount converted to local currency with the symbol positioned before or after the amount as appropriate','xoogu-currency-conversion');?></dd>
                            <dt>%asn</dt>
                            <dd><?php _e('The amount converted to local currency with the symbol in html numerical entity format positioned before or after the amount as appropriate','xoogu-currency-conversion');?></dd>
                            <dt>%asx</dt>
                            <dd><?php _e('The amount converted to local currency with the symbol in html hex entity format positioned before or after the amount as appropriate','xoogu-currency-conversion');?></dd>
                            <?php /*<dt>%ac</dt>
                            <dd><?php _e('The amount converted to local currency with the ISO currency code positioned before or after the amount as appropriate','xoogu-currency-conversion');?></dd>*/?>
                        </dl>
                        <p><?php _e('E.g. [currencyConversion to="GBP"]50 USD[/currencyConversion] with a format of \'%o (%a %c)\' would give \'50 USD (33.21 GBP)\'.','xoogu-currency-conversion');?></p>
                        <p><?php _e('Or [currencyConversion]USD 50[/currencyConversion] with a format of \'%o - %asy in %n\' would show \'USD 50 - 35.72 € in Euros\' to a visitor from Austria.','xoogu-currency-conversion');?></p>
                        <p><?php _e('Please note that the options to position the currency symbol or code before or after the amount as appropriate (%asy, %asn, %asx, %ac) will only do this if the amount to be converted to is determined from the visitor\'s country. In cases where you specify the currency to be converted to, the currency symbol will always be positioned to the left of the converted amount. This is because the symbol position is determined from the visitor\'s country and not the type of currency.','xoogu-currency-conversion');?></p>
                    </div>
                    <label for="format"><?php _e('Format','xoogu-currency-conversion')?>:</label>
                    <input type="text" name="options[format]" id="format" value="<?php echo $formValues['format'];?>" />
                </dd>
                <dt><?php _e('Number of decimal places to use','xoogu-currency-conversion');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('You can choose to specify the number of decimal places that will be used when displaying the converted amount. If empty then the amount of decimal places will be automatically decided based on the currency being converted to. (Most currencies use 2 decimal places).','xoogu-currency-conversion');?></p>
                    </div>
                    <label for="dec_places"><?php _e('dec_places:','xoogu-currency-conversion');?></label>
                    <input type="number" min="0" value="<?php if(isset($formValues['dec_places'])){ echo $formValues['dec_places'];}?>" name="options[dec_places]" id="dec_places" <?php if(isset($errors['dec_places'])){echo 'class="error"';}?>/>
                </dd>
                <dt><?php _e('Use static rates','xoogu-currency-conversion');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('You can choose to use static conversion rates that are saved as part of the post / page metadata. Then in 10 years time if someone vists a post you made today, they will see what the amount was in their local currency at the time the post was written, rather than at current conversion rates.','xoogu-currency-conversion');?></p>
                        <p><?php _e('Note that the rates are saved when a post containing the shortcode is saved. So if you draft a post with the shortcode in today, but publish it in a month\'s time, the rates saved will be from today. If you publish a post today without the shortcode, then edit the post a month later and add the shortcode, the rates would be saved at the time the shortcode is added.','xoogu-currency-conversion');?></p>
                        <p><?php printf(__('You can also enable this functionality on a per shortcode basis by adding the attribute %1$s. Or, if you set static rates for all shortcodes here, you can disable it on a per shortcode basis by adding the attribute %2$s.','xoogu-currency-conversion'), 'static=&quot;1&quot;', 'static=&quot;0&quot;');?></p>
                    </div>
                    <label for="static"><?php _e('Static:','xoogu-currency-conversion');?></label>
                    <input type="checkbox" value="1" name="options[static]" id="static"<?php if(!empty($formValues['static'])){echo ' checked="checked"';}?> />
                </dd>
                <dt><?php _e('Frequency to update exchange rates','xoogu-currency-conversion');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('The plugin will automatically fetch updated currency conversion rates from Yahoo Finance. You can specify how often the rates should be updated.','xoogu-currency-conversion');?></p>
                        <p><?php _e('Please note that the only options built-in to wordpress are hourly, daily, and twice daily. If you see any other options here, they have probably been added by a theme or plugin. You can use them, but they may stop working if the plugin / theme that added them is removed.','xoogu-currency-conversion');?></p>
                        <p><?php printf(__('Rates were last updated: %s','xoogu-currency-conversion'), date_i18n('r', get_site_option('xoogu_currency_conversion_ratesLastUpdated') ));?></p>
                    </div>
                    <label for="updateFreq"><?php _e('Update Frequency:','xoogu-currency-conversion');?></label>
                    <select id="updateFreq" name="options[updateFreq]">
                    <?php
                    $freqs = wp_get_schedules();
                    foreach($freqs as $key => $info){
                        echo '<option value="'.$key.'"'.($formValues['updateFreq']==$key ? ' selected="selected"' : '').'>'.$info['display'].'</option>';
                    }
                    ?></select>
                </dd>
                <dt><?php _e('Add ons','xoogu-currency-conversion');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('Add ons to add additional functionality such as sidebar widgets.');?></p>
                    </div>
                    <dl id="xcc-addOns">
                    <?php
                    
                    //get installed addOns
                    $addOns = $this->getAddOnsData(dirname(__FILE__).'/add-ons/');
                    foreach($addOns as $addOnFile => $addOn){
                        echo '<dt>'.$addOn['Name'].'</dt>';
                        echo'<dd>';
                        echo '<label for="'.$addOn['Name'].'-on">'.__('On', 'xoogu-currency-conversion').'</label>';
                        echo '<input type="radio" value="1" id="'.$addOn['Name'].'-on" name="options[addOns]['.$addOnFile.']" '.(!empty($this->options['addOns'][$addOnFile]) ? 'checked="checked"' : '').'/>';
                        echo '</dd><dd>';
                        echo '<label for="'.$addOn['Name'].'-off">'.__('Off', 'xoogu-currency-conversion').'</label>';
                        echo '<input type="radio" value="" id="'.$addOn['Name'].'-off" name="options[addOns]['.$addOnFile.']"  '.(empty($this->options['addOns'][$addOnFile]) ? 'checked="checked"' : '').'/>';
                        echo'</dd>';
                    }
                    
                    ?>
                    </dl>
                </dd>
            </dl>
        <?php
        //create the hidden nonce and referrer fields - http://codex.wordpress.org/Function_Reference/wp_nonce_field
        wp_nonce_field('save_settings', '_xoogu_currencyConversion_nonce');
        submit_button();
        ?>
        </form><?php
    }
    
    /**
     *Filter callback attached to plugin_action_links_(plugin filename) hook
     *Adds a settings link to the plugin in the plugins list
     *@param array $links The links to print for this plugin entry
     *@return array The links to print for this plugin entry
     */
    public function plugin_action_links($links){
        array_unshift($links, '<a href="'. get_admin_url(null, 'options-general.php?page=xoogu_currency_conversion') .'">'.__('Settings','xoogu-currency-conversion').'</a>');
        return $links;
    }
    
    
    /**
     *Gets a list of the installed addOns and their metadata
     *@param string $dir The directory to look in for the addOns
     *@return array Associative array of the addOns with their metadata
     */
    public function getAddOnsData($dir){
        $addOns = array();
        
        $handle = opendir($dir);
        while( false !== ($entry = readdir($handle)) ){
            if($entry == '.' || $entry == '..'){
                continue;
            }
            //if the addOn is located in its own directory, the addOn file should have the exact same name as the directory
            if(is_dir($dir.'/'.$entry)){
                $addOnFile = $entry.'/'.$entry.'.php';
            }elseif(is_file($dir.'/'.$entry)){
                $addOnFile = $entry;
            }
            if( $data = get_plugin_data( $dir.'/'.$addOnFile ) ){
                $addOns[$addOnFile]=$data;
            }
        }
        closedir($handle);
        
        return $addOns;
    }
    
    
    /**
     *Create the database table when the plugin is activated
     */
    public function activate($network){
        //check the current has permission to activate the plugin
        if(!current_user_can('activate_plugins')){
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-currency-conversion' ) );
        }
        
        //check the nonce is valid (will automatically die if it is not)
        $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
        //$plugin = activate-plugin_xoogu-currency-conversion/xoogu-currency-conversion.php
        check_admin_referer("activate-plugin_{$plugin}");
        
        global $wpdb;
        $wpdb->show_errors();
        $tblCur=$this->_tblCur;
        $tblCou=$this->_tblCou;
        
        
		//check the version and update the database table if necessary
		$version = get_site_option('xoogu_currency_conversion_version');
		if($version && $version !== '1.2'){
			//Data and table structure has been modified from previous version
			//Easiest thing to do is drop existing tables then make them anew
			
			//dbDelta doesn't support dropping tables (weird?) so use a standard query
			if( !$wpdb->query("DROP TABLE IF EXISTS `$tblCur`, `$tblCou`") ){
                //couldn't drop tables so die
                wp_die(new WP_Error('update', sprintf(__('Dropping of table %s or %s failed.','xoogu-currency-conversion'), $tblCur, $tblCou), $sql));
            }
		}
		//update the wordpress table with the version (useful for checking when upgrading)
		//we can use update as this will add the entry if doesn't already exist
		update_site_option('xoogu_currency_conversion_version', '1.2');
        
		//create the db tables and insert data
		
		//Currency details
        if($wpdb->get_var("SHOW TABLES LIKE '$tblCur'") != $tblCur) {
            //create the table
            $sql = 'CREATE TABLE IF NOT EXISTS '.$tblCur.' (
                    code CHAR(3) NOT NULL,
                    symbol VARCHAR(6) NOT NULL,
                    name VARCHAR(25) NOT NULL,
                    symbol_entity_num VARCHAR(30) NOT NULL,
                    symbol_entity_hex VARCHAR(30) NOT NULL,
                    full_name VARCHAR(50) NOT NULL,
                    dec_places TINYINT(1) NOT NULL,
                    symbol_pref BOOLEAN NOT NULL,
					format_pref CHAR(2) NOT NULL,
                    rate FLOAT UNSIGNED NOT NULL,
                    PRIMARY KEY  (code),
                    KEY (symbol)
            )';
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
            //unfortunately there seems no way to tell if dbDelta created the table successfully or not from its return value, so we'll have to manually check if the table was created
            if($wpdb->get_var("SHOW TABLES LIKE '$tblCur'") != $tblCur){
                //couldn't create table so die
                wp_die(new WP_Error('tblCurCreation', sprintf(__('Creation of table %s failed.','xoogu-currency-conversion'), $tblCur), $sql));
            }
            //Fill the table
            $sql= "INSERT INTO $tblCur (code, symbol, name, symbol_entity_num, symbol_entity_hex, full_name, dec_places, symbol_pref, format_pref) VALUES ";
            if (($handle = fopen(dirname(__FILE__).'/currency_details.csv', 'r')) !== FALSE) {
                $data = fgets($handle);
                $sql.= "($data)";
                while (($data = fgets($handle)) !== FALSE) {
                    $sql.= ", ($data)";
                }
                fclose($handle);
            }else{
                //couldn't open data file so die
                wp_die(new WP_Error('tblCurCreation', sprintf(__('Couldn\'t open data file %s.','xoogu-currency-conversion'), 'currency_details.csv') ));
            }
            if(!$wpdb->query($sql)){
                //couldn't fill table so die
                wp_die(new WP_Error('tblCurCreation', sprintf(__('Couldn\'t insert data into %s.','xoogu-currency-conversion'),$tblCur ),$sql ));
			}
        }
		
		//Country to currency map table
        if($wpdb->get_var("SHOW TABLES LIKE '$tblCou'") != $tblCou) {
            //create the table
            //symbol_pos - 0 => before amount, 1 => after amount
			//dec_point and thousands_sep must be varchar and not char as mysql trims char fields on retrieval (so space will end up as an empty string)
            $sql = 'CREATE TABLE IF NOT EXISTS '.$tblCou.' (
                    country CHAR(2) NOT NULL,
                    currency CHAR(3) NOT NULL,
                    dec_point VARCHAR(1) NOT NULL,
                    thousands_sep VARCHAR(1) NOT NULL,
                    symbol_pos BOOLEAN NOT NULL,
                    PRIMARY KEY  (country,currency)
            )';
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
            //If the table wasn't created then die
            if($wpdb->get_var("SHOW TABLES LIKE '$tblCou'") != $tblCou) {
                //couldn't create table so die
                wp_die(new WP_Error('tblCouCreation', sprintf(__('Creation of table %s failed.','xoogu-currency-conversion'), $tblCou), $sql));
            }
            //Fill the table
            $sql= "INSERT INTO $tblCou VALUES ";
            if (($handle = fopen(dirname(__FILE__).'/country_currency_map.csv', 'r')) !== FALSE) {
                $data = fgets($handle);
                $sql.= "($data)";
                while (($data = fgets($handle)) !== FALSE) {
                    $sql.= ", ($data)";
                }
                fclose($handle);
            }else{
                //couldn't open data file so die
                wp_die(new WP_Error('tblCouCreation', sprintf(__('Couldn\'t open data file %s.','xoogu-currency-conversion'), 'country_currency_map.csv') ));
            }
            
            if(!$wpdb->query($sql)){
                //couldn't fill table so die
                wp_die(new WP_Error('tblCouCreation', sprintf(__('Couldn\'t insert data into %s.','xoogu-currency-conversion'),$tblCou ),$sql ));
			}
        }
        
        //create the network-wide option to store when the cron was last run
		//we set this to 0 regardless of whether it already exists or not, so that activating the plugin will force a rate update
        update_site_option('xoogu_currency_conversion_ratesLastUpdated', 0);
        
        //multisite network activation
        if(is_multisite() && $network){
            if(!is_super_admin()){
                wp_die( __( 'You must be a super admin to network activate the plugin', 'xoogu-currency-conversion' ) );
            }
            $current = $wpdb->blogid;
            // Loop through the blogs
            $blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                $this->_activate_site();
            }
            //switch back to the original blog
            switch_to_blog($current);
        }
        //standard
        else{
            $this->_activate_site();
        }
    }
    
    
    /**
     *Creates the options entries in the db for the current site and schedules the cron for the current cite
     */
    protected function _activate_site(){
        //default format: original text (USD 50)
        if(empty($this->options)){
            $this->options = array('format' => '%o (%a %c)', 'updateFreq' => 'daily');
            if( ($geoFuncs=$this->detectGeoIP()) ){
                $this->options['geoFunc']=key($geoFuncs);
            }
        }
        add_option('xoogu_currency_conversion', $this->options);
        
        //update the rates regularly
        //get the registered cron frequency details
        $freqs = wp_get_schedules();
        //schedule the cron
        wp_schedule_event(time()+$freqs[$this->options['updateFreq']]['interval'],$this->options['updateFreq'],'xoogu_currency_conversion_update');
        //update the rates now (we don't use cron for the first update as it won't run straight away if using a real cron to trigger wp_cron)
        $this->updateRates();
    }
    
    
    /**
     *Callback attached to the wpmu_new_blog hook
     *When a new blog is added to a multisite network, we need to check whether the plugin should be activated for this new blog
     *We don't need the params other than $blog_id, though for param details see http://codex.wordpress.org/Function_Reference/wpmu_create_blog
     */
    function new_site_added($blog_id, $user_id, $domain, $path, $site_id, $meta ) {
        global $wpdb;
        //only activate the plugin for the new site if it has been network activated
        if (is_plugin_active_for_network(plugin_basename($this->mainPluginFile))) {
            $current = $wpdb->blogid;
            switch_to_blog($blog_id);
            //run the plugin activation for the new site
            $this->_activate_site();
            switch_to_blog($current);
        }
    }
    
    
    /**
     *Remove cron and all database records when plugin is removed
     */
    public static function uninstall(){
        
        //check permissions
        if(!current_user_can('activate_plugins')){
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-currency-conversion' ) );
        }
        //check nonce
        check_admin_referer('bulk-plugins');        
        
        global $wpdb;
        //mysql table names (copied from __construct of main class, can't use non static properties in uninstall function)
        $tblCur = $wpdb->base_prefix.'xoogu_currency_details';
        $tblCou = $wpdb->base_prefix.'xoogu_currency_country';
        
        //multisite network uninstall (can only ever be a network uninstall in multisite)
        if(is_multisite()){
            if(!is_super_admin()){
                wp_die( __( 'You must be a super admin to delete the plugin', 'xoogu-currency-conversion' ) );
            }
            $current = $wpdb->blogid;
            // Loop through the blogs
            $blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                self::_uninstall_site();
            }
            //switch back to the original blog
            switch_to_blog($current);
        }
        //standard
        else{
            self::_uninstall_site();
        }
        
        //remove the db tables
        $wpdb->query("DROP TABLE $tblCur");
        $wpdb->query("DROP TABLE $tblCou");
        
        //remove the ratesLastUpdated option used by the cron
        delete_site_option('xoogu_currency_conversion_ratesLastUpdated');
        //remove the plugin version from the db
        delete_site_option('xoogu_currency_conversion_version');
    }
    
    
    /**
     *Removes the per site database options used by the plugin
     */
    private static function _uninstall_site(){
        global $wpdb;
        
        //remove the update job from the cron
        self::_deactivate_site();
        
        //remove the options saved in the db
        delete_option('xoogu_currency_conversion');
        
        //remove any static rates saved as post_meta. We don't use a loop with delete_post_meta as suggested by the codex ( http://codex.wordpress.org/Function_Reference/delete_post_meta#Other_Examples ) as that would result in a lot of separate SQL queries and be slooow.
        if($wpdb->query('DELETE FROM '.$wpdb->postmeta." WHERE meta_key = '_xoogu_currency_conversion_rates'") === false){
            return new WP_Error('post_meta', __('Failed deleting post_meta', 'xoogu-currency-conversion'), $wpdb->last_error);
        }
    }
    
    
    /**
     *Remove cron when plugin is deactivated
     *@param bool $network If the plugin is being network deactivated
     */
    public function deactivate($network){
        //check the current has permission to activate the plugin
        if(!current_user_can('activate_plugins')){
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-currency-conversion' ) );
        }
        
        //check the nonce is valid (will automatically die if it is not)
        $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
        //$plugin = activate-plugin_xoogu-currency-conversion/xoogu-currency-conversion.php
        check_admin_referer( "deactivate-plugin_{$plugin}" );
        
        
        global $wpdb;
        
        //multisite
        if(is_multisite() && $network){
            if(!is_super_admin()){
                wp_die( __( 'You must be a super admin to network deactivate the plugin.', 'xoogu-currency-conversion' ) );
            }
            $current = $wpdb->blogid;
            // Loop through the blogs
            $blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                self::_deactivate_site();
            }
            //switch back to the original blog
            switch_to_blog($current);
        }
        //standard
        else{
            self::_deactivate_site();
        }
    }
    
    
    /**
     *Removes the per site cron
     */
    private static function _deactivate_site(){
        //remove the update job from the cron
        wp_clear_scheduled_hook('xoogu_currency_conversion_update');
    }
    
    
    /**
     *Gets the latest currency conversion rates from Yahoo and updates the database
     */
    public function updateRates(){
        //check the rates need updating
        //get the time the rates were last updated
        $lastRun = get_site_option('xoogu_currency_conversion_ratesLastUpdated');
        //get the registered cron frequency details
        $freqs = wp_get_schedules();
        //if the rates don't need updating yet (can happen if running the plugin on 2 or more sites in a multisite installation), then we don't need to do anything
        if(time() < $lastRun+$freqs[$this->options['updateFreq']]['interval']){
			return;
        }
        
        global $wpdb;
        $tblCur=$this->_tblCur;
		
		if ( ($get=wp_remote_get('http://finance.yahoo.com/webservice/v1/symbols/allcurrencies/quote?format=json')) && ($rateStr = wp_remote_retrieve_body($get)) ) {
			if($info = json_decode($rateStr)){
				
				//initialise an array to hold the values to be inserted
				$sqlParts=array();
				//get the currency codes from the database
				//we flip the array so we can check the code exists easily rather than having to use in_array()
				$codes = array_flip($this->getCurrencyList());
				//loop through the results from Yahoo
				foreach($info->list->resources as $currency){
					//parse the currency code (Yahoo data gives it like GBP=X, so we just need the first 3 chars to get the code)
					$code = substr($currency->resource->fields->symbol, 0, 3);
					//check the code is in the db
					if(isset($codes[$code])){
						//add the rate to the sql query
						$sqlParts[] = "('$code', ".(1/(float)$currency->resource->fields->price).") ";
					}
				}
				$sql = "INSERT INTO $tblCur (code, rate) VALUES ".implode(', ', $sqlParts).'ON DUPLICATE KEY UPDATE rate = VALUES(rate)';
				//update the database
				if($wpdb->query($sql) === false){
					return new WP_Error( 'updateRates', __('Failed updating the database table', 'xoogu-currency-conversion'), $wpdb->last_error );
				}
				//update the database to indicate the time the rates have been updated
				update_site_option('xoogu_currency_conversion_ratesLastUpdated', time());
			}
			else{
				return new WP_Error('updateRates', __('Failed decoding exchange rate json data from Yahoo', 'xoogu-currency-conversion'), $rateStr);
			}
		}
        else{
            return new WP_Error('updateRates', __('Failed fetching exchange rates from Yahoo', 'xoogu-currency-conversion'));
        }
    }
    
    
    /**
     *Saves the current rates as post meta for the current post
     */
    public function saveStaticRates($post_id){
        global $wpdb;
        $tblCur=$this->_tblCur;
        //fetch the codes and rates from the db
        if(!($results = $wpdb->get_results("SELECT code, rate FROM $tblCur"))){
            return;
        }
        //format as an associative array with the currency code as the key
        $rates = array();
        foreach($results as $row){
            $rates[$row->code]=$row->rate;
        }
        //save the post meta to the db
        add_post_meta($post_id, '_xoogu_currency_conversion_rates', $rates, true);
    }
    
    
    /**
     *Called when post is saved. Checks to see if we have any shortcodes with the static attribute set to true
     *If we do, then will save the current rates as post meta
     *@param int $post_id The ID of the post being saved
     */
    public function onPostSave($post_id){
        
        //don't bother doing anything if just an autosave
        if (defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE){
            return;
        }
        
        global $post_data;
        //the content being saved may be stored in $_POST, $_GET, or $post_data.
        //Probably $_GET is only used for quick edits, where you can't edit the content anyway, so no need to check it
        //WP adds slashes to POST data, so now we need to remove them GRR!!!
        $post_content = wp_unslash(!empty($_POST['post_content']) ? $_POST['post_content'] : $post_data['post_content']);
        
        //if there's no content, then no point in looking for the shortcode
        if(empty($post_content)){
            return;
        }
        
        //if the rates have already been saved in the post meta, then there's nothing to do
        if(get_post_meta($post_id, '_xoogu_currency_conversion_rates', true)){
            return;
        }
        
        //if static is set in the global options, then save the rates if the content includes the shortcode
        if(!empty($this->options['static'])){
            if(has_shortcode($post_content, 'currencyConversion')){
                return $this->saveStaticRates($post_id);
            }
            return;
        }
        
        //check if we have any shortcodes with the static attribute
        //has_shortcode won't let you check if an attribute is set, so have to use a custom regex:
        $pattern = '/(?<!\\[)\\[currencyConversion[^\\]]* static=[\'"]?([^\'"]+)[\'"]?(?:\\]| .*\\])/U';
        $exists = preg_match_all($pattern, $post_content, $matches);
        //check our matches
        foreach($matches[1] as $val){
            //if static was set to true (case insenstive) or 1, then save the rates to the post meta
            if(!empty($val) && strcasecmp($val, 'true') === 0 || $val == '1'){
                return $this->saveStaticRates($post_id);
            }
        }
    }
    
    
    /**
     *Detects installed methods of getting the country code of a visitor
     *@return array List of supported methods
     */
    public function detectGeoIP(){
        //cache result - switch_blog for WPMU does not modify plugins loaded so this func will always give the same result for each blog
        static $funcs;
        if(is_array($funcs)){
            return $funcs;
        }
        global $wpdb;
        $funcs=array();
        if(isset($_SERVER['GEOIP_COUNTRY_CODE'])){
            $funcs['server'] = __('Server has GeoIP extension installed','xoogu-currency-conversion');
        }
        //PHP GeoIP extension - http://www.php.net/manual/en/function.geoip-country-code-by-name.php
        if(extension_loaded('geoip')){
            $funcs['php'] = __('PHP GeoIP extension','xoogu-currency-conversion');
        }
        //GeoIP Detection plugin - http://wordpress.org/plugins/geoip-detect/
        if(is_plugin_active('geoip-detect/geoip-detect.php') && function_exists('geoip_detect_get_info_from_ip')){
            $funcs['geoip-detect'] = __('GeoIP Detection plugin','xoogu-currency-conversion');
        }
        //visitor-country - http://wordpress.org/plugins/visitor-country/
        if(is_plugin_active('visitor-country/visitor-country.php') && isset($GLOBALS['VisitorCountry'])){
            $funcs['visitor-country'] = __('Visitor Country plugin','xoogu-currency-conversion');
        }
        //iptocountry plugin - http://wordpress.org/plugins/ip-to-country/
        if(is_plugin_active('ip-to-country/ip-to-country.php') && class_exists('PepakIpToCountry')){
            $funcs['ip-to-country'] = __('IP to Country plugin','xoogu-currency-conversion');
        }
        //statPress - http://wordpress.org/plugins/statpress-visitors/
        if(is_plugin_active('statpress-visitors/statpress-visitors.php') && function_exists('geoip_open') && defined('GEOIP_STANDARD')){
            $funcs['statpress-visitors'] = __('Statpress Visitors plugin','xoogu-currency-conversion');
        }
        //Try ip2nation table - http://wordpress.org/plugins/wp-ip2nation-installer/
        if($wpdb->get_var("SHOW TABLES LIKE 'ip2nation'")) {
            $funcs['ip2nation'] = __('ip2nation database table','xoogu-currency-conversion');
        }
        //move the pointer to first item
        reset($funcs);
        return $funcs;
    }
    
    
    /**
     *Installs the Wordpress GeoIP Detection plugin
     */
    protected function _installGeoIP(){
        //check the user has the necessary privileges
        if(!current_user_can('activate_plugins')){
            return wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-currency-conversion' ) );
        }
        //get the path the plugin will be installed to
        $pluginPath=WP_PLUGIN_DIR.'/geoip-detect';
        $pluginFile=$pluginPath.'/geoip-detect.php';
        
        //if the plugin is already activated, then something has gone wrong
        if(is_plugin_active(plugin_basename($pluginFile))){
            throw new Exception(__('Hmm... The install GeoIP detection plugin button should only appear if the plugin is not already activated. But it appears that the plugin is already activated! How did this happen?!','xoogu-currency-conversion'));
            return;
        }
        
        //if the plugin doesn't exist, download it
        if(!is_dir($pluginPath)){
        
            //http://codex.wordpress.org/Filesystem_API
            //check we can write to the filesytem as the user - if we can't a form asking for FTP details will automatically be output
            $url = wp_nonce_url(get_admin_url(null, 'options-general.php?page=xoogu_currency_conversion'), 'installGeoIP');
            //form fields won't be passed correctly. This is a bug in WP - http://core.trac.wordpress.org/ticket/19643
            if (false === ($creds = request_filesystem_credentials($url, '', false, WP_PLUGIN_DIR, array('options' => array('from', 'dec_places', 'updateFreq', 'static', 'format'))) ) ) {
                return false; //If we can't write to the filesystem as the user, then don't continue
            }
            //Make sure the credentials actually work
            if ( ! WP_Filesystem($creds) ) {
                //If not, display the form asking for FTP details again
                request_filesystem_credentials($url, '', true, WP_PLUGIN_DIR, array('options' => array('from', 'dec_places', 'updateFreq', 'static', 'format')));
                return false;
            }
            
            //Use functions provided by wp-admin/includes/file.php
            //download the plugin
            $zip = download_url('http://downloads.wordpress.org/plugin/geoip-detect.zip');
            //unzip to the plugins directory
            $res = unzip_file($zip, WP_PLUGIN_DIR.'/');
            //delete the zip file
            unlink($zip);
        }
        $res = activate_plugin($pluginFile);
        if( is_wp_error($res) ){
            return $res;
        }
        //fire the update method of the plugin to fetch the GeoIP data
        //can be a bit slow so reset the script timeout limit
        set_time_limit(ini_get('max_execution_time'));
        geoip_detect_update();
        //update the plugin settings to use the GeoIp Detection WP plugin
        $this->options['geoFunc']='geoip-detect';
        update_option('xoogu_currency_conversion', $this->options);
        //update the POST so that the form will show with the GeoIp Detection WP plugin selected as the geoFunc method
        $_POST['options']['geoFunc']='geoip-detect';
        return true;
    }
	
    
}