<?php
class Xoogu_CurrencyConversion_Admin_MCE{
	
	protected $_pearant;
	
    /**
     *Initialise the plugin
     *@param Xoogu_CurrencyConversion_Admin $pearant The parent object this object will 'inherit' from
     */
    public function __construct(Xoogu_CurrencyConversion_Admin $pearant){
		$this->_pearant = $pearant;
		//tiny mce (call directly rather than hooking into admin_init as we are being loaded from the current_screen hook, which is called after the admin_init hook)
		$this->mce_init();
		//quicktags and shared
		add_action('admin_enqueue_scripts', array($this, 'loadAdminjscss'));
        add_action('wp_ajax_mce_popup', array($this, 'mce_popup'));
		add_action('wp_ajax_quicktags_dialog', array($this, 'quicktags_dialog'));
    }
    
	
	/**
	 *Loads the scripts and CSS needed to add the shortcode button and settings dialog for the tinyMCE and quicktags editors
	 */
	public function loadAdminjscss(){
		//quicktags
		//wordpress does include some jquery ui dialog css, but it is rather minimal and results in a transparaent div
		//wp_enqueue_style('wp-jquery-ui-dialog');
		wp_enqueue_style('jquery-ui', '//code.jquery.com/ui/1.10.3/themes/smoothness/jquery-ui.css');
		wp_enqueue_script('xoogu_currency_conversion_quicktags', plugins_url( 'quicktags.js' , __FILE__ ), array('jquery-ui-dialog', 'jquery','quicktags'), Xoogu_CurrencyConversion::$version, true );
		wp_enqueue_style( 'xoogu_currency_conversion_mce', plugins_url( 'mce_quicktags_shared.css' , __FILE__ ), null, Xoogu_CurrencyConversion::$version );
		
		//shared
		wp_enqueue_script('xoogu_currency_conversion_mce_quicktags_shared', plugins_url( 'mce_quicktags_shared.js' , __FILE__ ), array('jquery'), Xoogu_CurrencyConversion::$version, true );
		wp_localize_script( 'xoogu_currency_conversion_mce_quicktags_shared', 'objectL10n', array(
			'dec_places_error'  => __("dec_places must be an integer of zero or greater (or blank)", 'xoogu-currency-conversion' ),
			'content_error' => __("Content must include an amount to be converted", 'xoogu-currency-conversion' ),
			'show_description' => __('Show description', 'xoogu-currency-conversion'),
			'hide_description' => __('Hide description', 'xoogu-currency-conversion'),
			'help_for_setting' => __('Help for this setting', 'xoogu-currency-conversion'),
		    'button_title' => __('Show this amount to visitors in their local currency', 'xoogu-currency-conversion'),
			'mce_plugin_longname' => sprintf(__('%s Currency Conversion shortcode', 'xoogu-currency-conversion'), 'Xoogu'),
			'dialog_title'  => sprintf(__('%s Currency Conversion shortcode settings', 'xoogu-currency-conversion'), 'Xoogu')
		) );
	}
    
	
    /**
     *Adds the filters needed to add the button to the TinyMCE Editor
     */
    public function mce_init() {
        add_filter( 'mce_external_plugins', array($this, 'mce_add_buttons') );
        add_filter( 'mce_buttons', array($this,'mce_register_buttons') );
    }
    
	
    /**
     *Adds the TinyMCE plugin javascript
     *@param Array $plugin_array Array of TinyMCE plugins to be loaded
     *@return Array Array of TinyMCE plugins to be loaded
     */
    public function mce_add_buttons( $plugin_array ) {
        $plugin_array['xoogu_currency_conversion'] = plugins_url( 'mce-button.js' , __FILE__ );
        return $plugin_array;
    }
    
	
    /**
     *Registers the button to display in the TinyMCE Editor
     *@param Array $plugin_array Array of TinyMCE buttons to be available
     *@return Array Array of TinyMCE buttons to be available
     */
    function mce_register_buttons( $buttons ) {
        array_push( $buttons, 'xoogu_currency_conversion' );
        return $buttons;
    }
    
	
    /**
     *Display the settings page
     *See also the partial duplicate in Xoogu_CurrencyConversion_Admin::display_page
     */
    public function display_mce_form(){
		$pearant=$this->_pearant;
		?>
        <form autocomplete="off" id="xccsh_form" >
            <ul>
                <li>
					<div class="desc">
						<p><?php _e('The content that will be wrapped in the shortcode. This must include the amount to be converted','xoogu-currency-conversion');?></p>
					</div>
					<label for="xccsh_content"><?php _e('Content enclosed by the shortcode:','xoogu-currency-conversion');?></label>
                    <input type="text" value="" name="xccsh_content" id="xccsh_content" required="required" />
				</li>
				<li>
                    <div class="desc">
                        <p><?php _e('If you haven\'t set a default from currency, or you want to convert from a different currency to the default, then you can choose the currency to convert from. If left blank, then the shortcode will try to determine which currency to convert from automatically based on the text it encloses.','xoogu-currency-conversion');?></p>
                    </div>
                    <label for="xccsh_from"><?php _e('Convert from currency:','xoogu-currency-conversion');?></label>
                    <select name="xccsh_options[from]" id="xccsh_from">
                        <?php $codes = $pearant->getCurrencyList();
                        array_unshift($codes, '');
                        foreach($codes as $code){
                            echo '<option>'.$code.'</option>';
                        }?>
                    </select>
                </li>
				<li>
					<div class="desc">
						<p><?php _e('If you want to convert the amount to a specific currency, choose the currency to convert to here. Otherwise the currency will be automatically converted to the visitor\'s currency (assuming you have set up the Geo IP support).','xoogu-currency-conversion');?></p>
					</div>
					<label for="xccsh_to"><?php _e('Convert to currency:','xoogu-currency-conversion');?></label>
                    <select name="xccsh_options[to]" id="xccsh_to">
                        <?php 
                        foreach($codes as $code){
                            echo '<option>'.$code.'</option>';
                        }
                        unset($code,$codes);?>
                    </select>
				</li>
				<li>
					<div class="desc">
                        <p><?php _e('You can choose to specify the number of decimal places that will be used when displaying the converted amount. If empty then the default settings will be used (if set), otherwise the amount of decimal places will be automatically decided based on the currency being converted to. (Most currencies use 2 decimal places).','xoogu-currency-conversion');?></p>
                    </div>
                    <label for="xccsh_dec_places"><?php _e('Decimal places:','xoogu-currency-conversion');?></label>
                    <input type="number" min="0" name="xccsh_options[dec_places]" id="xccsh_dec_places" />
                </li>
                <li>
                    <div class="desc">
                        <p><?php _e('You can choose to use static conversion rates that are saved as part of the post / page metadata. Then in 10 years time if someone vists a post you made today, they will see what the amount was in their local currency at the time the post was written, rather than at current conversion rates.','xoogu-currency-conversion');?></p>
                        <p><?php _e('Note that the rates are saved when a post containing the shortcode is saved. So if you draft a post with the shortcode in today, but publish it in a month\'s time, the rates saved will be from today. If you publish a post today without the shortcode, then edit the post a month later and add the shortcode, the rates would be saved at the time the shortcode is added.','xoogu-currency-conversion');?></p>
                        <p><?php _e('If you have chosen to use static rates in the plugin settings, but you want this shortcode to always use up to date rates, then you can disable static rates for this shortcode by setting static to false.','xoogu-currency-conversion');?></p>
                    </div>
					<span><?php _e('Use static rates:','xoogu-currency-conversion');?></span>
                    <ul class="xccsh_static_list">
						<li>
							<label for="xccsh_static_default"><?php _e('Use default setting','xoogu-currency-conversion');?></label>
							<input type="radio" value="default" name="xccsh_options[static]" id="xccsh_static_default" checked="checked" />
						</li>
						<li>
							<label for="xccsh_static_true"><?php _e('True','xoogu-currency-conversion');?></label>
							<input type="radio" value="1" name="xccsh_options[static]" id="xccsh_static_true" />
						<li>
						<li>
							<label for="xccsh_static_false"><?php _e('False','xoogu-currency-conversion');?></label>
							<input type="radio" value="0" name="xccsh_options[static]" id="xccsh_static_false" />
						<li>
					</ul>
                </li>
                <li>
                    <div class="desc">
                        <p><?php _e('The format that will be used for replacing the text enclosed by the shortcode. Leave blank to use the default format specified in the plugin settings.','xoogu-currency-conversion');?></p>
						<p><?php _e('You can use the following replacements as part of the format:','xoogu-currency-conversion');?></p>
                        <dl>
                            <dt>%o</dt>
                            <dd><?php _e('Original text (the text enclosed by the shortcode)','xoogu-currency-conversion');?></dd>
                            <dt>%sy</dt>
                            <dd><?php _e('Currency symbol e.g. $','xoogu-currency-conversion');?></dd>
                            <dt>%sn</dt>
                            <dd><?php _e('Currency symbol in html numerical entity format','xoogu-currency-conversion');?></dd>
                            <dt>%sx</dt>
                            <dd><?php _e('Currency symbol in html hex entity format','xoogu-currency-conversion');?></dd>
                            <dt>%n</dt>
                            <dd><?php _e('Name of currency e.g. dollars','xoogu-currency-conversion');?></dd>
                            <dt>%fn</dt>
                            <dd><?php _e('Full name of currency e.g. United States Dollars','xoogu-currency-conversion');?></dd>
                            <dt>%c</dt>
                            <dd><?php _e('ISO currency code e.g. USD','xoogu-currency-conversion');?></dd>
                            <dt>%a</dt>
                            <dd><?php _e('The amount converted to local currency','xoogu-currency-conversion');?></dd>
                            <dt>%asy</dt>
                            <dd><?php _e('The amount converted to local currency with the symbol positioned before or after the amount as appropriate','xoogu-currency-conversion');?></dd>
                            <dt>%asn</dt>
                            <dd><?php _e('The amount converted to local currency with the symbol in html numerical entity format positioned before or after the amount as appropriate','xoogu-currency-conversion');?></dd>
                            <dt>%asx</dt>
                            <dd><?php _e('The amount converted to local currency with the symbol in html hex entity format positioned before or after the amount as appropriate','xoogu-currency-conversion');?></dd>
                        </dl>
                        <p><?php _e('E.g. [currencyConversion to="GBP"]50 USD[/currencyConversion] with a format of \'%o (%a %c)\' would give \'50 USD (33.21 GBP)\'.','xoogu-currency-conversion');?></p>
                        <p><?php _e('Or [currencyConversion]USD 50[/currencyConversion] with a format of \'%o - %asy in %n\' would show \'USD 50 - 35.72 € in Euros\' to a visitor from Austria.','xoogu-currency-conversion');?></p>
                        <p><?php _e('Please note that the options to position the currency symbol or code before or after the amount as appropriate (%asy, %asn, %asx, %ac) will only do this if the amount to be converted to is determined from the visitor\'s country. If you have specified the currency to be converted to, the currency symbol will always be positioned to the left of the converted amount. This is because the symbol position is determined from the visitor\'s country and not the type of currency.','xoogu-currency-conversion');?></p>
                    </div>
                    <label for="xccsh_format"><?php _e('Replacement format','xoogu-currency-conversion')?>:</label>
                    <input type="text" name="xccsh_options[format]" id="xccsh_format" />
                </li>
            </ul>
        </form><?php
    }
	
	
	/**
	 *Outputs the settings form the quicktags dialog AJAX call
	 */
	public function quicktags_dialog(){
		header('Content-type: text/html');
		$this->display_mce_form();
		exit();
	}
	
	
	/**
	 *Outputs an HTML page with the shortcode settings, called by the tinyMCE shortcode button
	 */
	public function mce_popup(){
		header('Content-type: text/html');
		wp_enqueue_script('jquery');
		//load the tiny MCE popup js (included with wordpress)
		//wp_enqueue_script('wpdialogs-popup'); //this doesn't work, seems not to just be a minified version of below
		wp_enqueue_script('tiny_mce_popup', get_site_url(null, '/wp-includes/js/tinymce/tiny_mce_popup.js'));
		//load our script for handling the settings form
		wp_enqueue_script('xccsh_mce_popup', plugins_url( 'mce_popup.js' , __FILE__ ), array('jquery'), Xoogu_CurrencyConversion::$version);
		wp_localize_script( 'xccsh_mce_popup', 'objectL10n', array(
			'cancel' => __('Cancel', 'xoogu-currency-conversion'),
			'insert' => __('Insert shortcode', 'xoogu-currency-conversion')
		) );
		//load the style for the settings form
		wp_enqueue_style( 'xoogu_currency_conversion_mce', plugins_url( 'mce_quicktags_shared.css' , __FILE__ ), null, Xoogu_CurrencyConversion::$version );
		
		
		?><!DOCTYPE html>
		<html>
		<head>
			<title><?php printf(__('%s Currency Conversion shortcode settings', 'xoogu-currency-conversion'), 'Xoogu');?></title>
			<?php wp_head();?>
		</head>
		
		<body>
		<?php
		$this->display_mce_form(); ?>
		<?php wp_footer();?>
		</body>
		</html>
		<?php exit();
	}
}