<?php
/*
Plugin Name: Choose currency widget
Plugin URI: http://www.xoogu.com/currency-conversion-wordpress-plugin/
Description: Widget gives a drop-down of currency codes from which the user can select their currency. This will then be used for currency conversions instead of guessing the user's currency from their IP address.
Version: 1.0.0
Author: xoogu.com
Author URI: http://www.xoogu.com/
License: GPLv2 or later
Text Domain: xoogu-currency-conversion
*/

class choose_currency_widget extends WP_Widget {
	 
	 public static $version='1.0.0';
	 
	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			'choose_currency_widget', // Base ID
			__('Currency Conversion choose currency', 'xoogu-currency-conversion'), // Name
			array( 'description' => __('Allows the user to manually choose their currency from a drop down menu. The selected currency will then be used for any currency conversions instead of basing the currency on the user\'s IP address.', 'xoogu-currency-conversion') ) // Args
		);
		
		add_action('init', array($this, 'init'));
	}
	
	/**
	 *Initialises the widget - starts session, processes submission if the widget was submitted, and modifies the currency to convert to if the user has manually chosen their currency
	 */
	public function init(){
		if( is_active_widget( null, null, 'choose_currency_widget', true ) ){
		  if (!session_id()){session_start();}
		  if(isset($_POST['xcc-code-selected'])){
			$this->save_selection($_POST['currencyCode']);
		  }
		  if(isset($_SESSION['xcc-code-selected'])){
			   add_filter('xcc_currencyTo', array($this, 'filterCurrencyTo'));
		  }
		}
	}
	
	/**
	 *Changes the provided currency code to that chosen by the user.
	 *Should be applied to 'xcc_currencyTo' filter
	 *@param string $to Currency code to convert to (generally this will be the one automatically selected based on the user's IP address)
	 *@return string The currency code to convert to (chosen by the user)
	 */
	public function filterCurrencyTo($to){
		  return $_SESSION['xcc-code-selected'];
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {
		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $args['before_widget'];
		if ( !empty( $title ) ){
			echo $args['before_title'] . $title . $args['after_title'];
		}
		
		global $Xoogu_CurrencyConversion;
		$currency = $Xoogu_CurrencyConversion->getCurrencyByIP();
		echo '<form action="'.$_SERVER["REQUEST_URI"].'" method="post">';
		echo '<select id="'.$args['widget_id'].'-select" name="currencyCode">
		<option value="">'.sprintf( __('Auto (%s)', 'xoogu-currency-conversion'), (empty($currency) ? __('Unknown', 'xoogu-currency-conversion') : $currency->code) ).'</option>';
		$currencyListFile=dirname(__FILE__).'/currency_select.html';
		if(file_exists($currencyListFile)){
		  $currencyList = file_get_contents($currencyListFile);
		}else{
		  //Generate <select> list of currencies
		  //get list of currencies
		  $codes = $Xoogu_CurrencyConversion->getCurrencyList();
		  sort($codes);
		  file_put_contents( $currencyListFile, ($currencyList = '<option>'.implode('</option><option>', $codes).'</option>') );
		}
		if(!empty($_SESSION['xcc-code-selected'])){
		  echo str_replace('>'.$_SESSION['xcc-code-selected'], ' selected="selected">'.$_SESSION['xcc-code-selected'], $currencyList );
		}else{
		  echo $currencyList;
		}
		echo '</select>
		<input type="submit" name="xcc-code-selected" value="Save" />
		</form>';
		
		echo $args['after_widget'];
	}
	
	
	/**
	 *Save the provided currency code as the selected code to the user's SESSION data
	 *@param string $code The 3 letter currency code to save as the selected currency for this user
	 */
	public function save_selection($code){
		  if(empty($code)){
			   unset($_SESSION['xcc-code-selected']);
			   return;
		  }
		  $_SESSION['xcc-code-selected'] = $code;
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		$title = isset( $instance[ 'title' ] ) ? $instance[ 'title' ] : __('Display amounts in this currency:', 'xoogu-currency-conversion');
		?>
		<p>
		<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>">
		</p>
		<?php 
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		$instance['title'] = ( !empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		return $instance;
	}
}




function register_choose_currency_widget(){
     register_widget( 'choose_currency_widget' );
}
add_action( 'widgets_init', 'register_choose_currency_widget');