jQuery(function($){
window['xccsh_shared'] = {
    //Reference to the settings form
    "form" : null,
    //object to hold any errors that occur
    "errors" : {},
    /**
     *adds a reference to the setting form so we access it easily from the other methods
     *@param jQuery $form jQuery object referencing the settings form
     *@returns xccsh_shared Returns this so method chaining can be used
     */
    "setForm" : function($form){
        this.form=$form;
        return this;
    },
    /**
     *Hides the desciptions for each of the settings
     *@returns xccsh_shared Returns this so method chaining can be used
     */
    "hideDescs" : function(){
        //add anchors like this with each so click function works with window loaded by TinyMCE
        this.form.find('div.desc').each(function(){
            $(this).hide();
            $('<a href="" class="desc-toggle" title="'+objectL10n.help_for_setting+'" >'+objectL10n.show_description+'</a>').click(
                function(e){
                    var $desc = $(this).next('div.desc');
                    if ($desc.css('display') == 'none') {
                        $(this).text(objectL10n.hide_description);
                        $desc.show();
                    }
                    else{
                        $(this).text(objectL10n.show_description);
                        $desc.hide();
                    }
                    e.preventDefault();
                    return false;
                }
            ).insertBefore(this);
        });
        return this;
    },
    /**
     *Validates the settings and returns the correct shortcode based on the settings
     *@returns bool|string Returns false if settings didn't validate, otherwise returns the shortcode
     */
    "getShortcode" : function(){
        //remove any error messages
		this.remove_errors();
 
		var replacement = '[currencyConversion',
		form = this.form,
        
        //text content
        selTextEl=form.find('#xccsh_content'),
        selText = selTextEl.val();
        if (!/\d/.test(selText)) {
            this.errors.content = {"el" : selTextEl, "msg" : objectL10n.content_error};
        }
        
        //from
        var from = form.find('#xccsh_from').val();
        if (from) {
            replacement += ' from="'+from+'"';
        }
        
        //to
        var to = form.find('#xccsh_to').val();
        if (to) {
            replacement += ' to="'+to+'"';
        }
		
		//dec_places
        var dec_placesEl = form.find('#xccsh_dec_places'),
		dec_places = dec_placesEl.val();
        if (dec_places !== '') {
			if (/^\d+$/.test(dec_places)) {
				replacement += ' dec_places="'+dec_places+'"';
			}
			else{
				this.errors.dec_places = {"el" : dec_placesEl, "msg" : objectL10n.dec_places_error};
        	}
		}
        
        //static
        var statik = form.find('input[name="xccsh_options[static]"]:checked').val();
        if (statik !== 'default') {
            replacement += ' static="'+statik+'"';
        }
        
        //format
        var format = form.find('#xccsh_format').val();
        if (format) {
            replacement += ' format="'+format+'"';
        }
		
		//check there weren't any errors
		if (!$.isEmptyObject(this.errors)) {
			this.add_errors();
			return false;
		}
        
        replacement += ']'+selText+'[/currencyConversion]';
        return replacement;
    },
    /**
     *Removes any error messages from the form
     *@returns void
     */
	"remove_errors" : function(){
		var errors = this.errors;
		
		//if there aren't any errors, there is nothing to do
		if ($.isEmptyObject(errors)) {
			return;
		}
		
		//remove the error css class from the affected items
		for(i in errors){
			errors[i].el.removeClass('error');
		}
		//reset the errors list
		this.errors={};
		//remove the errors list from the page
		this.form.find('#xccsh_errors').remove();
	},
	/**
     *Adds any error messages to the form
     *@returns void
     */
	"add_errors" : function(){
		var errors = this.errors;
		
		//if there aren't any errors, there is nothing to do
		if ($.isEmptyObject(errors)) {
			return;
		}
		
		var errorHTML = '';
		//add the error css class from the affected items
		for(i in errors){
			errors[i].el.addClass('error');
			errorHTML +='<li><a href="#'+errors[i].el.attr('id')+'">'+errors[i].msg+'</a></li>';
		}
		//add the errors list to the page
		this.form.animate({ "scrollTop": 0 });
		this.form.prepend('<div id="xccsh_errors"><ul>'+errorHTML+'</ul></div>').focus();
	}
}
});